@@ -22,8 +22,6 @@ from six.moves import urllib_parse
 
 from zaqar.common import auth
 from zaqar.common import urls
-from zaqar.i18n import _LE
-from zaqar.i18n import _LI
 from zaqar.storage import pooling
 
 LOG = logging.getLogger(__name__)
@@ -47,6 +45,7 @@ class NotifierDriver(object):
         max_workers = kwargs.get('max_notifier_workers', 10)
         self.executor = futurist.ThreadPoolExecutor(max_workers=max_workers)
         self.require_confirmation = kwargs.get('require_confirmation', False)
+        self.queue_controller = kwargs.get('queue_controller')
 
     def post(self, queue_name, messages, client_uuid, project=None):
         """Send messages to the subscribers."""
@@ -54,11 +53,14 @@ class NotifierDriver(object):
             if not isinstance(self.subscription_controller,
                               pooling.SubscriptionController):
                 marker = None
+                queue_metadata = self.queue_controller.get(queue_name,
+                                                           project)
+                retry_policy = queue_metadata.get('_retry_policy', {})
                 while True:
                     subscribers = self.subscription_controller.list(
                         queue_name, project, marker=marker)
                     for sub in next(subscribers):
-                        LOG.debug("Notifying subscriber %r" % (sub,))
+                        LOG.debug("Notifying subscriber %r", (sub,))
                         s_type = urllib_parse.urlparse(
                             sub['subscriber']).scheme
                         # If the subscriber doesn't contain 'confirmed', it
@@ -67,17 +69,18 @@ class NotifierDriver(object):
                         # should allow them be subscribed.
                         if (self.require_confirmation and
                                 not sub.get('confirmed', True)):
-                            LOG.info(_LI('The subscriber %s is not '
-                                         'confirmed.'), sub['subscriber'])
+                            LOG.info('The subscriber %s is not '
+                                     'confirmed.', sub['subscriber'])
                             continue
                         for msg in messages:
                             msg['Message_Type'] = MessageType.Notification.name
-                        self._execute(s_type, sub, messages)
+                        self._execute(s_type, sub, messages,
+                                      retry_policy=retry_policy)
                     marker = next(subscribers)
                     if not marker:
                         break
         else:
-            LOG.error(_LE('Failed to get subscription controller.'))
+            LOG.error('Failed to get subscription controller.')
 
     def send_confirm_notification(self, queue, subscription, conf,
                                   project=None, expires=None,
@@ -93,8 +96,8 @@ class NotifierDriver(object):
 
         key = conf.signed_url.secret_key
         if not key:
-            LOG.error(_LE("Can't send confirm notification due to the value of"
-                          " secret_key option is None"))
+            LOG.error("Can't send confirm notification due to the value of"
+                      " secret_key option is None")
             return
         url = "/%s/queues/%s/subscriptions/%s/confirm" % (api_version, queue,
                                                           subscription['id'])
@@ -113,12 +116,12 @@ class NotifierDriver(object):
         messages = {}
         endpoint_dict = auth.get_public_endpoint()
         if endpoint_dict:
-            wsgi_endpoint = endpoint_dict.get('zaqar', None)
+            wsgi_endpoint = endpoint_dict.get('zaqar')
             if wsgi_endpoint:
                 wsgi_subscribe_url = urllib_parse.urljoin(
                     wsgi_endpoint, url)
                 messages['WSGISubscribeURL'] = wsgi_subscribe_url
-            websocket_endpoint = endpoint_dict.get('zaqar-websocket', None)
+            websocket_endpoint = endpoint_dict.get('zaqar-websocket')
             if websocket_endpoint:
                 websocket_subscribe_url = urllib_parse.urljoin(
                     websocket_endpoint, url)
@@ -133,13 +136,14 @@ class NotifierDriver(object):
                          'SubscribeBody': {'confirmed': True},
                          'UnsubscribeBody': {'confirmed': False}})
         s_type = urllib_parse.urlparse(subscription['subscriber']).scheme
-        LOG.info(_LI('Begin to send %(type)s confirm/unsubscribe notification.'
-                     ' The request body is %(messages)s'),
+        LOG.info('Begin to send %(type)s confirm/unsubscribe notification.'
+                 ' The request body is %(messages)s',
                  {'type': s_type, 'messages': messages})
 
         self._execute(s_type, subscription, [messages], conf)
 
-    def _execute(self, s_type, subscription, messages, conf=None):
+    def _execute(self, s_type, subscription, messages, conf=None,
+                 retry_policy=None):
         if self.subscription_controller:
             data_driver = self.subscription_controller.driver
             conf = data_driver.conf
@@ -149,4 +153,4 @@ class NotifierDriver(object):
                                    s_type,
                                    invoke_on_load=True)
         self.executor.submit(mgr.driver.execute, subscription, messages,
-                             conf=conf)
+                             conf=conf, queue_retry_policy=retry_policy)
