@@ -309,19 +309,21 @@ class Queue(ControllerBase):
     numbers of queues.
     """
 
-    def list(self, project=None, marker=None,
-             limit=DEFAULT_QUEUES_PER_PAGE, detailed=False):
+    def list(self, project=None, kfilter={}, marker=None,
+             limit=DEFAULT_QUEUES_PER_PAGE, detailed=False, name=None):
         """Base method for listing queues.
 
         :param project: Project id
+        :param kfilter: The key-value of metadata which user want to filter
         :param marker: The last queue name
         :param limit: (Default 10) Max number of queues to return
         :param detailed: Whether metadata is included
+        :param name: The queue name which user want to filter
 
         :returns: An iterator giving a sequence of queues
             and the marker of the next page.
         """
-        return self._list(project, marker, limit, detailed)
+        return self._list(project, kfilter, marker, limit, detailed, name)
 
     _list = abc.abstractmethod(lambda x: None)
 
