@@ -14,8 +14,12 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import os
+
 import collections
 import datetime
+import hashlib
+import json
 import math
 import random
 import time
@@ -25,6 +29,7 @@ import ddt
 import mock
 from oslo_utils import timeutils
 import six
+import testtools
 from testtools import matchers
 
 from zaqar.common import cache as oslo_cache
@@ -68,15 +73,27 @@ class ControllerBaseTest(testing.TestBase):
                 args.append(self.control)
             self.driver = self.driver_class(*args)
         else:
-            uri = self.mongodb_url
-            for i in range(4):
-                db_name = "zaqar_test_pools_" + str(i)
+            testredis = os.environ.get('ZAQAR_TEST_REDIS', 0)
+            if testredis:
+                uri = self.redis_url
+                for i in range(4):
+                    db_name = "?dbid = " + str(i)
 
-                # NOTE(dynarro): we need to create a unique uri.
-                uri = "%s/%s" % (uri, db_name)
-                options = {'database': db_name}
-                self.control.pools_controller.create(six.text_type(i),
-                                                     100, uri, options=options)
+                    # NOTE(dynarro): we need to create a unique uri.
+                    new_uri = "%s/%s" % (uri, db_name)
+                    self.control.pools_controller.create(six.text_type(i),
+                                                         100, new_uri)
+            else:
+                uri = self.mongodb_url
+                for i in range(4):
+                    db_name = "zaqar_test_pools_" + str(i)
+
+                    # NOTE(dynarro): we need to create a unique uri.
+                    new_uri = "%s/%s" % (uri, db_name)
+                    options = {'database': db_name}
+                    self.control.pools_controller.create(six.text_type(i),
+                                                         100, new_uri,
+                                                         options=options)
             self.driver = self.driver_class(self.conf, cache, self.control)
             self.addCleanup(self.control.pools_controller.drop_all)
             self.addCleanup(self.control.catalogue_controller.drop_all)
@@ -432,8 +449,8 @@ class MessageControllerTest(ControllerBaseTest):
         # Test Message Get
         message_out = self.controller.get(queue_name, message_id,
                                           project=self.project)
-        self.assertEqual({'id', 'body', 'ttl', 'age', 'claim_id'},
-                         set(message_out))
+        self.assertEqual({'id', 'body', 'ttl', 'age', 'claim_count',
+                          'claim_id'}, set(message_out))
         self.assertEqual(message_id, message_out['id'])
         self.assertEqual(message['body'], message_out['body'])
         self.assertEqual(message['ttl'], message_out['ttl'])
@@ -445,6 +462,39 @@ class MessageControllerTest(ControllerBaseTest):
         with testing.expect(errors.DoesNotExist):
             self.controller.get(queue_name, message_id, project=self.project)
 
+    def test_message_body_checksum(self):
+        self.conf.enable_checksum = True
+        queue_name = self.queue_name
+        message = {
+            'ttl': 60,
+            'body': {
+                'event': 'BackupStarted',
+                'backupId': 'c378813c-3f0b-11e2-ad92-7823d2b0f3ce'
+            }
+        }
+
+        # Test Message Creation
+        created = list(self.controller.post(queue_name, [message],
+                                            project=self.project,
+                                            client_uuid=uuid.uuid4()))
+        self.assertEqual(1, len(created))
+        message_id = created[0]
+
+        # Test Message Get
+        message_out = self.controller.get(queue_name, message_id,
+                                          project=self.project)
+        self.assertEqual({'id', 'body', 'ttl', 'age', 'claim_count',
+                          'claim_id', 'checksum'}, set(message_out))
+
+        algorithm, checksum = message_out['checksum'].split(':')
+        expected_checksum = ''
+        if algorithm == 'MD5':
+            md5 = hashlib.md5()
+            md5.update(json.dumps(message['body']).encode('utf-8'))
+            expected_checksum = md5.hexdigest()
+
+        self.assertEqual(expected_checksum, checksum)
+
     def test_get_multi(self):
         client_uuid = uuid.uuid4()
 
@@ -488,8 +538,8 @@ class MessageControllerTest(ControllerBaseTest):
                                                 project=self.project)
 
         for idx, message in enumerate(messages_out):
-            self.assertEqual({'id', 'body', 'ttl', 'age', 'claim_id'},
-                             set(message))
+            self.assertEqual({'id', 'body', 'ttl', 'age', 'claim_count',
+                              'claim_id'}, set(message))
             self.assertEqual(idx, message['body'])
 
         self.controller.bulk_delete(self.queue_name, ids,
@@ -740,6 +790,7 @@ class MessageControllerTest(ControllerBaseTest):
 
         self.assertEqual(1, len(popped_messages))
 
+    @testtools.skip("Skip until fixed bug 1739332")
     def test_message_period(self):
         self.queue_controller.create(self.queue_name, project=self.project)
         messages = [
@@ -1051,6 +1102,90 @@ class ClaimControllerTest(ControllerBaseTest):
         self.assertEqual(9999, expected_msg["ttl"])
         self.assertEqual({"key": "value"}, expected_msg["body"])
 
+    def test_delay_queue(self):
+        meta = {'ttl': 1, 'grace': 0}
+        # Set default message delay for queue.
+        metadata = {'_default_message_delay': 2}
+        self.queue_controller.set_metadata(self.queue_name,
+                                           metadata,
+                                           project=self.project)
+        # Scenarios 1: Normal message sent to delayed queue
+        new_messages = [{'ttl': 3600, 'body': {'key': 'value'}}]
+        queue_meta = self.queue_controller.get_metadata(self.queue_name,
+                                                        project=self.project)
+        delay = queue_meta.get('_default_message_delay', 0)
+        for msg in new_messages:
+            if delay and 'delay' not in new_messages:
+                msg['delay'] = delay
+        ids = self.message_controller.post(self.queue_name, new_messages,
+                                           client_uuid=str(uuid.uuid1()),
+                                           project=self.project)
+        interaction = self.message_controller.list(self.queue_name,
+                                                   project=self.project,
+                                                   include_delayed=True)
+        delay_messages = list(next(interaction))
+        self.assertEqual(1, len(list(delay_messages)))
+
+        claim_id, messages = self.controller.create(self.queue_name, meta,
+                                                    project=self.project)
+        self.assertIsNone(claim_id)
+        self.assertEqual(0, len(list(messages)))
+        time.sleep(2)
+        claim_id, messages = self.controller.create(self.queue_name, meta,
+                                                    project=self.project)
+        self.assertIsNotNone(claim_id)
+        self.assertEqual(1, len(list(messages)))
+        time.sleep(1)
+        self.message_controller.delete(self.queue_name, ids[0],
+                                       project=self.project)
+
+        # Scenarios 2: Delay message sent to delayed queue
+        new_messages = [{'ttl': 3600, 'delay': 1,
+                         'body': {'key': 'value'}}]
+
+        ids = self.message_controller.post(self.queue_name, new_messages,
+                                           client_uuid=str(uuid.uuid1()),
+                                           project=self.project)
+        interaction = self.message_controller.list(self.queue_name,
+                                                   project=self.project,
+                                                   include_delayed=True)
+        delay_messages = list(next(interaction))
+        self.assertEqual(1, len(list(delay_messages)))
+
+        claim_id, messages = self.controller.create(self.queue_name, meta,
+                                                    project=self.project)
+        self.assertIsNone(claim_id)
+        self.assertEqual(0, len(list(messages)))
+        time.sleep(1)
+        claim_id, messages = self.controller.create(self.queue_name, meta,
+                                                    project=self.project)
+        self.assertIsNotNone(claim_id)
+        self.assertEqual(1, len(list(messages)))
+        time.sleep(1)
+        self.message_controller.delete(self.queue_name, ids[0],
+                                       project=self.project)
+
+        # Scenarios 3: Message sent to normal queue
+        new_messages = [{'ttl': 3600, 'body': {'key': 'value'}}]
+        self.queue_controller.set_metadata(self.queue_name,
+                                           {},
+                                           project=self.project)
+        ids = self.message_controller.post(self.queue_name, new_messages,
+                                           client_uuid=str(uuid.uuid1()),
+                                           project=self.project)
+        interaction = self.message_controller.list(self.queue_name,
+                                                   project=self.project,
+                                                   include_delayed=True)
+        delay_messages = list(next(interaction))
+        self.assertEqual(1, len(list(delay_messages)))
+        claim_id, messages = self.controller.create(self.queue_name, meta,
+                                                    project=self.project)
+        self.assertIsNotNone(claim_id)
+        self.assertEqual(1, len(list(messages)))
+        time.sleep(1)
+        self.message_controller.delete(self.queue_name, ids[0],
+                                       project=self.project)
+
 
 @ddt.ddt
 class SubscriptionControllerTest(ControllerBaseTest):
@@ -1217,6 +1352,39 @@ class SubscriptionControllerTest(ControllerBaseTest):
                                                    project=self.project)
         self.assertIsNone(s_id)
 
+    def test_create_existed_and_get_correct_id(self):
+        queue_name1 = 'test_queue1'
+        queue_name2 = 'test_queue2'
+        subscriber = 'http://fake'
+        self.queue_controller.create(queue_name1, project=self.project)
+        self.queue_controller.create(queue_name2, project=self.project)
+
+        self.subscription_controller.create(queue_name1,
+                                            subscriber,
+                                            self.ttl,
+                                            self.options,
+                                            project=self.project)
+
+        s_id_2 = self.subscription_controller.create(queue_name2,
+                                                     subscriber,
+                                                     self.ttl,
+                                                     self.options,
+                                                     project=self.project)
+
+        s_id_3 = self.subscription_controller.create(queue_name2,
+                                                     subscriber,
+                                                     self.ttl,
+                                                     self.options,
+                                                     project=self.project)
+        self.assertIsNone(s_id_3)
+
+        s_id = self.subscription_controller. \
+            get_with_subscriber(queue_name2,
+                                subscriber,
+                                project=self.project)['id']
+
+        self.assertEqual(str(s_id_2), s_id)
+
     def test_get_update_delete_on_non_existing_queue(self):
         self._precreate_queue(precreate_queue=True)
         s_id = self.subscription_controller.create(
@@ -1321,7 +1489,6 @@ class SubscriptionControllerTest(ControllerBaseTest):
                         s_id, self.project)
         subscription = self.subscription_controller.get(self.source, s_id,
                                                         project=self.project)
-
         self.assertFalse(subscription['confirmed'])
 
         self.subscription_controller.confirm(self.source, s_id,
@@ -1329,9 +1496,15 @@ class SubscriptionControllerTest(ControllerBaseTest):
                                              confirmed=True)
         subscription = self.subscription_controller.get(self.source, s_id,
                                                         project=self.project)
-
         self.assertTrue(subscription['confirmed'])
 
+        self.subscription_controller.confirm(self.source, s_id,
+                                             project=self.project,
+                                             confirmed=False)
+        subscription = self.subscription_controller.get(self.source, s_id,
+                                                        project=self.project)
+        self.assertFalse(subscription['confirmed'])
+
     def test_confirm_with_nonexist_subscription(self):
         s_id = 'fake-id'
         self.assertRaises(errors.SubscriptionDoesNotExist,
@@ -1356,10 +1529,12 @@ class PoolsControllerTest(ControllerBaseTest):
 
         # Let's create one pool
         self.pool = str(uuid.uuid1())
+        # NOTE(gengchc2): remove pool_group in Rocky release.
         self.pool_group = str(uuid.uuid1())
-        self.pools_controller.create(self.pool, 100, 'localhost',
-                                     group=self.pool_group, options={})
-
+        self.pool1 = str(uuid.uuid1())
+        self.flavor = str(uuid.uuid1())
+        self.pools_controller.create(self.pool1, 100, 'localhost1',
+                                     flavor=self.flavor, options={})
         self.flavors_controller = self.driver.flavors_controller
 
     def tearDown(self):
@@ -1425,6 +1600,9 @@ class PoolsControllerTest(ControllerBaseTest):
 
     def test_delete_works(self):
         self.pools_controller.delete(self.pool)
+        # (gengchc): Remove the flavor from pool, then testcase cleanup pool
+        self.pools_controller.update(self.pool1, flavor="")
+        self.pools_controller.delete(self.pool1)
         self.assertFalse(self.pools_controller.exists(self.pool))
 
     def test_delete_nonexistent_is_silent(self):
@@ -1437,9 +1615,9 @@ class PoolsControllerTest(ControllerBaseTest):
         self.assertRaises(StopIteration, next, pools)
 
     def test_listing_simple(self):
-        # NOTE(cpp-cabrera): base entry interferes with listing results
-        self.pools_controller.delete(self.pool)
-
+        # (gengchc): Remove the flavor from pool, then testcase cleanup pool
+        self.pools_controller.update(self.pool1, flavor="")
+        self.pools_controller._drop_all()
         pools = []
         marker = ''
         for i in range(15):
@@ -1502,15 +1680,6 @@ class PoolsControllerTest(ControllerBaseTest):
             self.assertIn('options', entry)
             self.assertEqual({}, entry['options'])
 
-    def test_mismatching_capabilities(self):
-        # NOTE(flaper87): This may fail for redis. Create
-        # a dummy store for tests.
-        with testing.expect(errors.PoolCapabilitiesMismatch):
-            self.pools_controller.create(str(uuid.uuid1()),
-                                         100, 'redis://localhost',
-                                         group=self.pool_group,
-                                         options={})
-
 
 class CatalogueControllerTest(ControllerBaseTest):
     controller_base_class = storage.CatalogueBase
@@ -1523,10 +1692,13 @@ class CatalogueControllerTest(ControllerBaseTest):
         self.project = six.text_type(uuid.uuid4())
 
         self.pool = str(uuid.uuid1())
-        self.pool_group = str(uuid.uuid1())
-        self.pool_ctrl.create(self.pool, 100, 'localhost',
-                              group=self.pool_group, options={})
+        self.pool_ctrl.create(self.pool, 100, 'localhost', options={})
         self.addCleanup(self.pool_ctrl.delete, self.pool)
+        self.pool1 = str(uuid.uuid1())
+        self.flavor = str(uuid.uuid1())
+        self.pool_ctrl.create(self.pool1, 100, 'localhost1',
+                              options={})
+        self.addCleanup(self.pool_ctrl.delete, self.pool1)
 
     def tearDown(self):
         self.controller.drop_all()
@@ -1581,8 +1753,9 @@ class CatalogueControllerTest(ControllerBaseTest):
 
     def test_update(self):
         p2 = u'b'
+        # NOTE(gengchc2): Remove [group=self.pool_group] in
+        # it can be tested for redis as management.
         self.pool_ctrl.create(p2, 100, '127.0.0.1',
-                              group=self.pool_group,
                               options={})
         self.addCleanup(self.pool_ctrl.delete, p2)
 
@@ -1637,6 +1810,8 @@ class CatalogueControllerTest(ControllerBaseTest):
         self.controller.insert(self.project, q2, u'a')
 
 
+# NOTE(gengchc2): remove FlavorsControllerTest in Rocky release
+# and use FlavorsControllerTest1 instead for pool_group removal.
 class FlavorsControllerTest(ControllerBaseTest):
     """Flavors Controller base tests.
 
@@ -1814,6 +1989,185 @@ class FlavorsControllerTest(ControllerBaseTest):
             self.assertEqual({}, entry['capabilities'])
 
 
+# NOTE(gengchc2): Unittest for new flavor configure scenario.
+class FlavorsControllerTest1(ControllerBaseTest):
+    """Flavors Controller base tests.
+
+    NOTE(flaper87): Implementations of this class should
+    override the tearDown method in order
+    to clean up storage's state.
+    """
+    controller_base_class = storage.FlavorsBase
+
+    def setUp(self):
+        super(FlavorsControllerTest1, self).setUp()
+        self.pools_controller = self.driver.pools_controller
+        self.flavors_controller = self.driver.flavors_controller
+
+        # Let's create one pool
+        self.pool = str(uuid.uuid1())
+        self.flavor = 'durable'
+        self.pools_controller.create(self.pool, 100, 'localhost',
+                                     options={})
+        self.addCleanup(self.pools_controller.delete, self.pool)
+
+    def tearDown(self):
+        self.pools_controller.update(self.pool, flavor="")
+        self.flavors_controller.drop_all()
+        super(FlavorsControllerTest1, self).tearDown()
+
+    def test_create_succeeds(self):
+        self.flavors_controller.create(self.flavor,
+                                       project=self.project,
+                                       capabilities={})
+
+    def _flavors_expects(self, flavor, xname, xproject):
+        self.assertIn('name', flavor)
+        self.assertEqual(xname, flavor['name'])
+        self.assertNotIn('project', flavor)
+
+    def test_create_replaces_on_duplicate_insert(self):
+        name = str(uuid.uuid1())
+        self.flavors_controller.create(name,
+                                       project=self.project,
+                                       capabilities={})
+        entry = self.flavors_controller.get(name, self.project)
+        self._flavors_expects(entry, name, self.project)
+        new_capabilities = {'fifo': False}
+        self.flavors_controller.create(name,
+                                       project=self.project,
+                                       capabilities=new_capabilities)
+        entry = self.flavors_controller.get(name, project=self.project,
+                                            detailed=True)
+        self._flavors_expects(entry, name, self.project)
+        self.assertEqual(new_capabilities, entry['capabilities'])
+
+    def test_get_returns_expected_content(self):
+        name = 'durable'
+        capabilities = {'fifo': True}
+        self.flavors_controller.create(name,
+                                       project=self.project,
+                                       capabilities=capabilities)
+        res = self.flavors_controller.get(name, project=self.project)
+        self._flavors_expects(res, name, self.project)
+        self.assertNotIn('capabilities', res)
+
+    def test_detailed_get_returns_expected_content(self):
+        name = 'durable'
+        capabilities = {'fifo': True}
+        self.flavors_controller.create(name,
+                                       project=self.project,
+                                       capabilities=capabilities)
+        res = self.flavors_controller.get(name, project=self.project,
+                                          detailed=True)
+        self._flavors_expects(res, name, self.project)
+        self.assertIn('capabilities', res)
+        self.assertEqual(capabilities, res['capabilities'])
+
+    def test_get_raises_if_not_found(self):
+        self.assertRaises(errors.FlavorDoesNotExist,
+                          self.flavors_controller.get, 'notexists')
+
+    def test_exists(self):
+        self.flavors_controller.create('exists',
+                                       project=self.project,
+                                       capabilities={})
+        self.assertTrue(self.flavors_controller.exists('exists',
+                                                       project=self.project))
+        self.assertFalse(self.flavors_controller.exists('notexists',
+                                                        project=self.project))
+
+    def test_update_raises_assertion_error_on_bad_fields(self):
+        self.assertRaises(AssertionError, self.pools_controller.update,
+                          self.flavor)
+
+    def test_update_works(self):
+        name = 'yummy'
+        self.flavors_controller.create(name,
+                                       project=self.project,
+                                       capabilities={})
+
+        res = self.flavors_controller.get(name, project=self.project,
+                                          detailed=True)
+
+        p = 'olympic'
+        flavor = name
+        self.pools_controller.create(p, 100, 'localhost2',
+                                     flavor=flavor, options={})
+        self.addCleanup(self.pools_controller.delete, p)
+
+        new_capabilities = {'fifo': False}
+        self.flavors_controller.update(name, project=self.project,
+                                       capabilities={'fifo': False})
+        res = self.flavors_controller.get(name, project=self.project,
+                                          detailed=True)
+        self._flavors_expects(res, name, self.project)
+        self.assertEqual(new_capabilities, res['capabilities'])
+        self.pools_controller.update(p, flavor="")
+
+    def test_delete_works(self):
+        name = 'puke'
+        self.flavors_controller.create(name,
+                                       project=self.project,
+                                       capabilities={})
+        self.flavors_controller.delete(name, project=self.project)
+        self.assertFalse(self.flavors_controller.exists(name))
+
+    def test_delete_nonexistent_is_silent(self):
+        self.flavors_controller.delete('nonexisting')
+
+    def test_drop_all_leads_to_empty_listing(self):
+        self.flavors_controller.drop_all()
+        cursor = self.flavors_controller.list()
+        flavors = next(cursor)
+        self.assertRaises(StopIteration, next, flavors)
+        self.assertFalse(next(cursor))
+
+    def test_listing_simple(self):
+        name_gen = lambda i: chr(ord('A') + i)
+        for i in range(15):
+            pool = str(i)
+            flavor = name_gen(i)
+            uri = 'localhost:2701' + pool
+            self.pools_controller.create(pool, 100, uri,
+                                         flavor=flavor, options={})
+            self.addCleanup(self.pools_controller.delete, pool)
+
+            self.flavors_controller.create(flavor, project=self.project,
+                                           capabilities={})
+
+        def get_res(**kwargs):
+            cursor = self.flavors_controller.list(project=self.project,
+                                                  **kwargs)
+            res = list(next(cursor))
+            marker = next(cursor)
+            self.assertTrue(marker)
+            return res
+
+        res = get_res()
+        self.assertEqual(10, len(res))
+        for i, entry in enumerate(res):
+            self._flavors_expects(entry, name_gen(i), self.project)
+            self.assertNotIn('capabilities', entry)
+
+        res = get_res(limit=5)
+        self.assertEqual(5, len(res))
+
+        res = get_res(marker=name_gen(3))
+        self._flavors_expects(res[0], name_gen(4), self.project)
+
+        res = get_res(detailed=True)
+        self.assertEqual(10, len(res))
+        for i, entry in enumerate(res):
+            self._flavors_expects(entry, name_gen(i), self.project)
+            self.assertIn('capabilities', entry)
+            self.assertEqual({}, entry['capabilities'])
+        # (gengchc): Remove the flavor from pool, then testcase cleanup pools
+        for i in range(15):
+            pool = str(i)
+            self.pools_controller.update(pool, flavor="")
+
+
 def _insert_fixtures(controller, queue_name, project=None,
                      client_uuid=None, num=4, ttl=120):
     def messages():
